/*
 * Copyright (c) 1987 by Silicon Graphics, Inc.
 *		Glen Williams
 */

/*		The Font Manager Client Interface		*/

#ifndef FMCLIENT
#define FMCLIENT

#ifdef __cplusplus
extern "C" {
#endif

typedef long *fmfonthandle;

typedef struct fmfontinfo {
	long printermatched;
	long reserved0;
	double matrix00;
	double matrix01;
	double matrix10;
	double matrix11;
	long type;		/* see fmfontinfo->type  below */
	long encoding;
	long fixed_width;
	long xorig;
	long yorig;
	long xsize;
	long ysize;
	long height;
	long nglyphs;
	long bitsdeep;	    /* if nonzero, the depth of all glyphs in font */
	long width;	    /* the largest setwidth in the font */
	long resolution;
	long weight;	    /* [0..1000], the # of black bits that would be on
			       if this were an of 1000-pixel chars */
	long reserved1;
	long reserved2;
	long reserved3;
	long padding[27];   /* for future expansion */
} fmfontinfo;


typedef struct fmglyphinfo {
	long xsize, ysize;		/* dimensions of the glyph in pixels */
	long xorig, yorig;		/* origin */
	float xmove, ymove;		/* move  */
	long gtype;			/* The type of glyph this is. */
	long bitsdeep;			/* depth of the pixels, if pixels */
}fmglyphinfo;


/* ******************************************************/
/* Overall font information subscripts: fmfontinfo	*/
/* ******************************************************/

#define FMFPRINTERMATCHED 0
#define FMFMATRIX00 1
#define FMFMATRIX01 2
#define FMFMATRIX10 3
#define FMFMATRIX11 4
#define FMFTYPE	    5
#define FMFENCODING 6
#define FMFFIXED_WIDTH 7
#define FMFXORIG 8
#define FMFYORIG 9
#define FMFXSIZE 10
#define FMFYSIZE 11
#define FMFNGLYPHS 12
#define FMFBITSDEEP 13
#define FMFWIDTH 14
#define FMFRESOLUTION 15
#define	FMFWEIGHT 16

/* enumeration of fmfontinfo->type */
#define FMBITMAPFONT	    0
#define FMWIDTHFONT	    1
#define FMPRINTERWIDTHFONT	2
#define FMSPLINEFONT	    3
#define FMVECTORFONT	    4
#define FMGREYSCALEFONT	    5
#define FMRGBFONT	    6
#define FMGREEKFONT	    7
#define FMPERSPECTIVEFONT   8
#define FMCOLORMAP	    9
#define FMMIXED		    10

/* enumeration of fmfontinfo->encoding */
#define FMASCII		0
#define FMADOBE		1
#define FMJISC6226	2
#define FMCYRILLIC	3
#define FMHANGUL	4
#define FMDEVENAGERI	5



/* *****************************************************/
/* Character glyph information subscripts: fmglyphinfo */
/* *****************************************************/
#define FMXSIZE	0
#define FMYSIZE	1
#define FMXORIG	2
#define FMYORIG	3
#define FMXMOVE	4
#define FMYMOVE	5
#define FMGTYPE	6
#define FMBITSDEEP	7


/* fmglyphinfo->gtype */
#define FMNOTDEFINED	    0
#define FMBITMAPGLYPH	    1
#define FMIMAGERGBGLYPH    2
#define FMIMAGECGLYPH	    3
#define FMLINEGLYPH	    4
#define FMSHAPEGLYPH	    5

#define FMCACHE_QUANTUM 100000

#if defined(mips) || defined(__cplusplus)
int		fmcachelimit();
void		fmconcatpagematrix(double [3][2]);
#ifdef	__cplusplus
void		fminit();
#ifdef	__cplusplus_compiler_wasnt_buggy
void		fmEnumerateFamilies(void (*func)(char *));
#endif
#else
/* no args declared because this C compiler can't handle them. */
void		fmenumerate();
void		fminit(void);
#endif
fmfonthandle	fmfindfont(const char *);
void		fmfreefont(fmfonthandle);
char		*fmfontpath(); /* pass back the path */
void		fmsetpath(const char *);/* pass in the path */
int		fmgetcacheused();
long		fmgetchrwidth(fmfonthandle, const unsigned char);
int		fmgetcomment(fmfonthandle, int, char *);
int		fmgetfontinfo(fmfonthandle, fmfontinfo *);
int		fmgetfontname(fmfonthandle, int len, char *);
void		fmgetpagematrix(double [3][2]);
long		fmgetstrwidth(fmfonthandle, const char *);
long		fmgetwholemetrics(fmfonthandle, fmglyphinfo *);
void		fminitpagematrix();
fmfonthandle	fmmakefont(fmfonthandle, double[3][2]);
void		fmcacheenable();
void		fmcachedisable();
long		fmoutchar(fmfonthandle, const unsigned int);
void		fmprintermatch(int);
int		fmprstr(const char *);
void		fmrotatepagematrix(double);
fmfonthandle	fmscalefont(fmfonthandle, double);
void		fmsetcachelimit(int);
void		fmsetfont(fmfonthandle);
void		fmsetpagematrix(double [3][2]);
void		fmscalepagematrix(double);
int		fmfprstr(fmfonthandle, unsigned char *, short, int, int);

#ifdef __cplusplus
/* temporary __cplusplus functions - NOT SUPPORTED */
long		fmOutString(unsigned char *ch, short len, int x, int y);
void		fmForceFixedWidth(fmfonthandle);
#endif

#else	/* not mips or __cplusplus */

int		fmcachelimit();
void		fminit();

void		fmenumerate();
fmfonthandle	fmfindfont();
fmfonthandle	fmscalefont();
void		fmsetfont();

void		fmconcatpagematrix();
int		fmgetcacheused();
void		fmgetpagematrix();
void		fminitpagematrix();
void		fmrotatepagematrix();
void		fmscalepagematrix();
void		fmsetcachelimit();
void		fmsetpagematrix();

char		*fmfontpath();
void		fmsetpath();
void		fmfreefont();

long		fmgetchrwidth();
int		fmgetcomment();
int		fmgetfontinfo();
int		fmgetfontname();
long		fmgetstrwidth();
int		fmgetwholemetrics();

fmfonthandle	fmmakefont();
void		fmprintermatch();

void		fmcacheenable();
void		fmcachedisable();
long		fmoutchar();
int		fmprstr();
int		fmfprstr();
#endif

#ifdef __cplusplus
}
#endif

#endif
