#ifndef __LOCALE_H__
#define __LOCALE_H__
#ifdef __cplusplus
extern "C" {
#endif
/**************************************************************************
 *									  *
 * 		 Copyright (C) 1984, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

/* #ident "$Revision: 1.4 $" */

/* ANSI C Notes:
 *
 * - THE IDENTIFIERS APPEARING OUTSIDE OF #ifdef __EXTENSIONS__ IN THIS
 *   standard header ARE SPECIFIED BY ANSI!  CONFORMANCE WILL BE ALTERED
 *   IF ANY NEW IDENTIFIERS ARE ADDED TO THIS AREA UNLESS THEY ARE IN ANSI's
 *   RESERVED NAMESPACE. (i.e., unless they are prefixed by __[a-z] or
 *   _[A-Z].  For external objects, identifiers with the prefix _[a-z] 
 *   are also reserved.)
 *
 * - Macro definitions beginning with LC_[A-Z] are also reserved when 
 *   including <locale.h>.
 *
 */

/* struct lconv contains information related to the formatting of numeric
   values.  The semantics of each member of the structure is given below
   along with the default value in the "C" locale.
*/

#ifndef NULL
#define NULL	0
#endif

struct lconv {
	/* The decimal-point character used to format non-monetary quantities */
	char *	decimal_point;		/* "." */

	/* The character used to separate groups of digits before the	*/
	/* decimal-point character in formatted non-monetary quantities */
	char *	thousands_sep;		/* "" */

	/* A string whose elements indicate the size of each group of 	*/
	/* digits in formatted non-monetary quantities.  		*/
	char *	grouping;

	/* The international currency symbol applicable to the current	*/
	/* locale.  The first three characters are specified in ISO4217.*/
	/* (see X3.159-1989 sec.4.4.2.1)				*/
	char *	int_curr_symbol;
	char *	currency_symbol;
	char *	mon_decimal_point;
	char *	mon_thousands_sep;
	char * 	mon_grouping;
	char *	positive_sign;
	char *	negative_sign;
	/* unnamed pads. */
	int	:32,:32;

	char 	int_frac_digits;
	char 	frac_digits;
	char 	p_cs_precedes;
	char	p_sep_by_space;
	char	n_cs_precedes;
	char	n_sep_by_space;
	char	p_sign_posn;
	char	n_sign_posn;
	/* unnamed pads. */
	int	:32,:32;
};

typedef struct lconv __lconv_t;

#define LC_ALL		1
#define LC_MIN		LC_ALL
#define LC_COLLATE	2
#define LC_CTYPE	3
#define LC_MONETARY	4
#define LC_NUMERIC	5
#define LC_TIME		6
#define LC_MAX		LC_TIME

/* functions */
char *		setlocale(int category, const char * locale);
struct lconv *	localeconv(void);

#ifdef __cplusplus
}
#endif
#endif /* __LOCALE_H__ */

