/*	@(#)mntent.h	1.3 88/05/20 4.0NFSSRC SMI;	from SMI 1.13 99/01/06	*/

/*
 * File system table, see mntent (5)
 *
 * Used by dump, mount, umount, swapon, fsck, df, ...
 *
 * Quota files are always named "quotas", so if type is "rq",
 * then use concatenation of mnt_dir and "quotas" to locate
 * quota file.
 */

#define	MNTTAB		"/etc/fstab"
#define	MOUNTED		"/etc/mtab"

#define	MNTMAXSTR	128

#include <sys/fsid.h>

#define MNTTYPE_EFS	FSID_EFS	/* extent filesystem */
#define MNTTYPE_NFS	FSID_NFS	/* network file system */
#define MNTTYPE_SOCKET	FSID_SOCKET	/* socket pseudo-filesystem */
#define MNTTYPE_BELL	S51K		/* bell (att s5) filesystem */
#define MNTTYPE_DBG	FSID_DBG	/* debug (proc) pseudo-filesystem */
#define MNTTYPE_PIPE	FSID_COM	/* pipe pseudo-filesystem */
#define	MNTTYPE_PC	"pc"		/* IBM PC (MSDOS) file system */
#define	MNTTYPE_SWAP	"swap"		/* swap file system */
#define	MNTTYPE_IGNORE	"ignore" 	/* No type specified; ignore entry */
#define MNTTYPE_LO      "lo"    	/* Loop back File system */
#define	MNTTYPE_DOS	"dos"		/* MS-DOS file system (SGI) */

#define	MNTOPT_RO	"ro"		/* read only */
#define	MNTOPT_RW	"rw"		/* read/write */
#define	MNTOPT_QUOTA	"quota"		/* quotas */
#define	MNTOPT_NOQUOTA	"noquota"	/* no quotas */
#define	MNTOPT_NOSUID	"nosuid"	/* disallow setuid program execution */
#define MNTOPT_NODEV	"nodev"		/* disallow access to device files */
#define	MNTOPT_NOAUTO	"noauto"	/* hide entry from mount -a */
#define MNTOPT_GRPID 	"grpid"		/* SysV-compatible group-id on create */
#define MNTOPT_REMOUNT	"remount"	/* change options on previous mount */
#define MNTOPT_RAW	"raw"		/* raw device name */
#define MNTOPT_FSCK	"fsck"		/* fsck by default */
#define MNTOPT_NOFSCK	"nofsck"	/* do not fsck */
#define	MNTOPT_SOFT	"soft"		/* soft mount */
#define	MNTOPT_HARD	"hard"		/* hard mount */
#define	MNTOPT_INTR	"intr"		/* allow interrupts on hard mount */
#define	MNTOPT_NOAC	"noac"		/* don't cache nfs attributes */
#define MNTOPT_PORT	"port"		/* server IP port number */
#define MNTOPT_RETRANS	"retrans"	/* set number of request retries */
#define MNTOPT_RSIZE	"rsize"		/* set read size (bytes) */
#define MNTOPT_WSIZE	"wsize"		/* set write size (bytes) */
#define MNTOPT_TIMEO	"timeo"		/* set initial timeout (1/10 sec) */
#define MNTOPT_ACTIMEO	"actimeo"	/* attr cache timeout (sec) */
#define MNTOPT_ACREGMIN	"acregmin"	/* min ac timeout for reg files (sec) */
#define MNTOPT_ACREGMAX	"acregmax"	/* max ac timeout for reg files (sec) */
#define MNTOPT_ACDIRMIN	"acdirmin"	/* min ac timeout for dirs (sec) */
#define MNTOPT_ACDIRMAX	"acdirmax"	/* max ac timeout for dirs (sec) */
#define MNTOPT_PRIVATE	"private"	/* mount nfs single-client tree */
#define MNTOPT_SYMTTL	"symttl"	/* symlink cache time-to-live */

#define MNTINFO_DEV     "dev"   /* device number of the mounted file system */

struct mntent {
	char	*mnt_fsname;		/* name of mounted file system */
	char	*mnt_dir;		/* file system path prefix */
	char	*mnt_type;		/* MNTTYPE_* */
	char	*mnt_opts;		/* MNTOPT* */
	int	mnt_freq;		/* dump frequency, in days */
	int	mnt_passno;		/* pass number on parallel fsck */
};

#ifdef __cplusplus
extern "C" {
#endif

struct __file_s	*setmntent(const char *, const char *);
struct mntent	*getmntent(struct __file_s *);
int		addmntent(struct __file_s *, struct mntent *);
char	*hasmntopt(struct mntent *, const char *);
int		endmntent(struct __file_s *);

#ifdef __cplusplus
}
#endif
