#if IP17
#ifndef __SYS_IP17ADDRS_H__
#define __SYS_IP17ADDRS_H__

/**************************************************************************
 *									  *
 * 		 Copyright (C) 1990, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
#ident "$Revision: 1.10 $"

/*
 * Definitions of physical memory usage for the IP17
 * 
 * Physical memory usage for the IP17
 * 
 * 				---------------------------------
 * 0x1fc00000			| Prom text & read only data	|
 * 				---------------------------------
 * 0x00580000 (5.5M)		| Grinch/Venice gfx PROM (copy)	|
 * 				---------------------------------
 * 0x00580000 (5.5M)		| Prom stack			|
 *				---------------------------------
 * 0x00500000 (5M)		| Prom bss			|
 *				---------------------------------
 * 0x00400000 (4M)		| Sash				|
 *				---------------------------------
 * 0x003ffffc (4M-4)		| Sash Stack			|
 *				---------------------------------
 * 0x00054000			| Debug version of UNIX		|
 * 				---------------------------------
 * 0x00004010			| Symmon			|
 * 				---------------------------------
 * 0x00004000			| Symmon Stack			|
 * 				---------------------------------
 * 0x00003000			| UNIX				|
 * 				---------------------------------
 * 0x00001000			| GM Communication area (8k)	|
 * 				---------------------------------
 * 0x00001000			| eframe for Cache Err handler	|
 * 				---------------------------------
 * 0x00001000-eframe size	| stack for Cache Err handler	|
 * 				---------------------------------
 * 0x00000500			| MPCONF block (unused by kern.)|
 * 				---------------------------------
 * 0x00000400			| restart block			|
 * 				---------------------------------
 * 0x00000180			| general exception handler	|
 * 				---------------------------------
 * 0x00000100			| Cache error handler		|
 * 				---------------------------------
 * 0x00000080			| Xutlbmiss handler (not used)	|
 * 				---------------------------------
 * 0x00000000			| utlbmiss handler		|
 * 				---------------------------------
 *
 * *********************** NOTE *************************
 * To allow two SKYWRITER pipes in a system, the 8Kbytes GM
 * communication area is now different.  The change is ugly
 * but hopefully will allow two GMs to coexist without requiring
 * modification to the CPU PROM.
 *
 * The 8Kbyte area for VGX(1 pipe), starting at address 0x1000:
 *	2Kb 	host to GM communication area.
 * 	2Kb	GM to host communication area.
 *	1Kb	area for GM powerup diagnostics.
 *	2Kb	host and GM shared area.
 *	1Kb	unused.
 * The new scheme is as follow:
 *	2Kb	host to GM communication area for pipe 0.
 * 	2Kb	GM to host communication area for pipe 0.
 *	1Kb	area for GM powerup diagnostics for pipe 0.
 *	1Kb	host and GM shared area for pipe 0.
 *	1Kb	host and GM shared area for pipe 1.
 *	1Kb	area for GM powerup diagnostics for pipe 1.
 * The 2Kb host to GM communication area and the 2Kb GM to host
 * communication area for pipe 1 are allocated by the kernel
 * at driver init time.  The physical address of this memory
 * is stored by the host at a known place in the host and GM shared area
 * for pipe 1.  When the GM of pipe 1 gets an init memory command,
 * it reads in this address and uses it for communication from then
 * on.
 */ 

/*
 * Address definitions
 */
#define PROM_BSS_END		0xa0600000
#define GFX_PROM_COPY		0xa0580000
#define PROM_STACK		0xa0580000
#define PROM_BSS		0xa0500000
#define	SYMMON_STACK		0xa0004000
#define	SYMMON_STACKEND		SYMMON_STACK
#define	GMCOMM_ADDR		0xa0001000
#define	MPCONF_ADDR		0xa0000500
#define	RESTART_ADDR		0xa0000400
#define PROM_CHILD_STACK	0x803ffffc

/* Location of the eframe and stack for the ecc handler.
 * Since the R4000 'replaces' KUSEG with an unmapped, uncached space
 * corresponding to physical memory when a cache error occurs, these are
 * the actual addresses used.
 */
#define CACHE_ERR_EFRAME	(0x1000 - EF_SIZE)
#define CACHE_ERR_ECCFRAME	(CACHE_ERR_EFRAME - ECCF_SIZE)

/* 4 arguments on the stack. */
#define CACHE_ERR_SP		(CACHE_ERR_ECCFRAME - 4 * sizeof(int))

#ifdef PON_UNCACHED
#define	PON_STACKADDR		(0xa0000000+(1024*1024*6))	/* 6 meg */
#else
#define	PON_STACKADDR		(0x80000000+(1024*1024*6))
#endif /* PON_UNCACHED */

/*
 * Size definitions
 */
#define GMCOMM_SIZE		8*1024
#define	SYMMON_PDASIZE		512	/* size per CPU */

#endif	/* __SYS_IP17ADDRS_H__ */
#endif	/* IP17 */
