#ifndef __SYS_ERRNO_H__
#define __SYS_ERRNO_H__

/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*#ident	"@(#)kern-port:sys/errno.h	10.7.1.1"*/
/* #ident	"$Revision: 3.14 $" */

/* ANSI C Notes:
 *
 *  - This header is included by the ANSI header <errno.h>, thus is 
 *    governed by the standard.  
 *    		DO NOT MAKE ANY CHANGES TO THIS HEADER 
 *	      UNLESS YOU CONSULT THE ANSI STANDARD ON C 
 *		   (X3.159-1989, sec. 4.1.3.)
 *
 * - THE IDENTIFIERS APPEARING OUTSIDE OF #ifdef __EXTENSIONS__ IN THIS
 *   standard header ARE SPECIFIED BY ANSI!  CONFORMANCE WILL BE ALTERED
 *   IF ANY NEW IDENTIFIERS ARE ADDED TO THIS AREA UNLESS THEY ARE IN ANSI's
 *   RESERVED NAMESPACE. (i.e., unless they are prefixed by __[a-z] or
 *   _[A-Z].  For external objects, identifiers with the prefix _[a-z] 
 *   are also reserved.)
 *
 *  - Macros that begin with E and either a digit or an upper-case
 *    letter are reserved when including errno.h.
 *
 *  - Definitions segmenting the error number space have been replicated 
 *    with a versions prefixed with 'E' a la ANSI, with the non-prefixed 
 *    version appearing under __EXTENSIONS__.
 */

/*
 * Error codes
 */

#define	EPERM	1	/* No permission match			*/
#define	ENOENT	2	/* No such file or directory		*/
#define	ESRCH	3	/* No such process			*/
#define	EINTR	4	/* Interrupted system call		*/
#define	EIO	5	/* I/O error				*/
#define	ENXIO	6	/* No such device or address		*/
#define	E2BIG	7	/* Arg list too long			*/
#define	ENOEXEC	8	/* Exec format error			*/
#define	EBADF	9	/* Bad file number			*/
#define	ECHILD	10	/* No child processes			*/
#define	EAGAIN	11	/* Resource temporarily unavailable	*/
#define	ENOMEM	12	/* Not enough space			*/
#define	EACCES	13	/* Permission denied			*/
#define	EFAULT	14	/* Bad address				*/
#define	ENOTBLK	15	/* Block device required		*/
#define	EBUSY	16	/* Device or resource busy		*/
#define	EEXIST	17	/* File exists				*/
#define	EXDEV	18	/* Cross-device link			*/
#define	ENODEV	19	/* No such device			*/
#define	ENOTDIR	20	/* Not a directory			*/
#define	EISDIR	21	/* Is a directory			*/
#define	EINVAL	22	/* Invalid argument			*/
#define	ENFILE	23	/* Too many open files in system	*/
#define	EMFILE	24	/* Too many open files in a process	*/
#define	ENOTTY	25	/* Inappropriate IOCTL operation	*/
#define	ETXTBSY	26	/* Text file busy			*/
#define	EFBIG	27	/* File too large			*/
#define	ENOSPC	28	/* No space left on device		*/
#define	ESPIPE	29	/* Illegal seek				*/
#define	EROFS	30	/* Read-only file system		*/
#define	EMLINK	31	/* Too many links			*/
#define	EPIPE	32	/* Broken pipe				*/
#define	EDOM	33	/* Argument out of domain 		*/
#define	ERANGE	34	/* Result too large			*/
#define	ENOMSG	35	/* No message of desired type		*/
#define	EIDRM	36	/* Identifier removed			*/
#define	ECHRNG	37	/* Channel number out of range		*/
#define	EL2NSYNC 38	/* Level 2 not synchronized		*/
#define	EL3HLT	39	/* Level 3 halted			*/
#define	EL3RST	40	/* Level 3 reset			*/
#define	ELNRNG	41	/* Link number out of range		*/
#define	EUNATCH 42	/* Protocol driver not attached		*/
#define	ENOCSI	43	/* No CSI structure available		*/
#define	EL2HLT	44	/* Level 2 halted			*/
#define	EDEADLK	45	/* Deadlock situation detected/avoided	*/
#define	ENOLCK	46	/* No record locks available.		*/

/* Convergent Error Returns */
#define EBADE	50	/* Bad exchange descriptor		*/
#define EBADR	51	/* Bad request descriptor		*/
#define EXFULL	52	/* Message tables full			*/
#define ENOANO	53	/* Anode table overflow			*/
#define EBADRQC	54	/* Bad request code			*/
#define EBADSLT	55	/* Invalid slot				*/
#define EDEADLOCK 56	/* File locking deadlock 		*/

#define EBFONT	57	/* Bad font file format			*/

/* stream problems */
#define ENOSTR	60	/* Not a stream device			*/
#define ENODATA	61	/* No data available			*/
#define ETIME	62	/* Timer expired			*/
#define ENOSR	63	/* Out of streams resources		*/

#define ENONET	64	/* Machine is not on the network	*/
#define ENOPKG	65	/* Package not installed                */
#define EREMOTE	66	/* Object is remote			*/
#define ENOLINK	67	/* Link has been severed		*/
#define EADV	68	/* Advertise error			*/
#define ESRMNT	69	/* Srmount error			*/

#define	ECOMM	70	/* Communication error on send		*/
#define EPROTO	71	/* Protocol error			*/
#define	EMULTIHOP 74	/* Multihop attempted			*/
#define EBADMSG 77	/* Not a data message			*/

#define ENOTUNIQ 80	/* Name not unique on network		*/
#define EBADFD	 81	/* File descriptor in bad state		*/
#define EREMCHG	 82	/* Remote address changed		*/

/* shared library problems */
#define ELIBACC	83	/* Can not access a needed shared lib.	*/
#define ELIBBAD	84	/* Accessing a corrupted shared lib.	*/
#define ELIBSCN	85	/* .lib section in a.out corrupted.	*/
#define ELIBMAX	86	/* Attempting to link in more shared libraries than system limit */
#define ELIBEXEC 87	/* Can not exec a shared library directly*/
#define	ENOSYS	88	/* IRIX uses EINVAL; POSIX wants ENOSYS	*/


/*
 * Networking error numbers (tcp/ip)
 */
#define ETCP_EBASE 100
/* non-blocking and interrupt i/o */
#define EWOULDBLOCK	(1+ETCP_EBASE)	/* Operation would block */
#define EINPROGRESS	(2+ETCP_EBASE)	/* Operation now in progress */
#define EALREADY	(3+ETCP_EBASE)	/* Operation already in progress */

/* argument errors */
#define ENOTSOCK	(4+ETCP_EBASE)	/* Socket operation on non-socket */
#define EDESTADDRREQ	(5+ETCP_EBASE)	/* Destination address required */
#define EMSGSIZE	(6+ETCP_EBASE)	/* Message too long */
#define EPROTOTYPE	(7+ETCP_EBASE)	/* Protocol wrong type for socket */
#define ENOPROTOOPT	(8+ETCP_EBASE)	/* Option not supported by protocol */
#define EPROTONOSUPPORT	(9+ETCP_EBASE)	/* Protocol not supported */
#define ESOCKTNOSUPPORT	(10+ETCP_EBASE)	/* Socket type not supported */
#define EOPNOTSUPP	(11+ETCP_EBASE)	/* Op not supported on socket */
#define EPFNOSUPPORT	(12+ETCP_EBASE)	/* Protocol family not supported */
#define EAFNOSUPPORT	(13+ETCP_EBASE)	/* Address family not supported by protocol family */
#define EADDRINUSE	(14+ETCP_EBASE)	/* Address already in use */
#define EADDRNOTAVAIL	(15+ETCP_EBASE)	/* Can't assign requested address */

/* operational errors */
#define ENETDOWN	(16+ETCP_EBASE)	/* Network is down */
#define ENETUNREACH	(17+ETCP_EBASE)	/* Network is unreachable */
#define ENETRESET	(18+ETCP_EBASE)	/* Network dropped connection on reset */
#define ECONNABORTED	(19+ETCP_EBASE)	/* Software caused connection abort */
#define ECONNRESET	(20+ETCP_EBASE)	/* Connection reset by peer */
#define ENOBUFS 	(21+ETCP_EBASE)	/* No buffer space available */
#define EISCONN 	(22+ETCP_EBASE)	/* Socket is already connected */
#define ENOTCONN	(23+ETCP_EBASE)	/* Socket is not connected */
#define ESHUTDOWN	(24+ETCP_EBASE)	/* Can't send after socket shutdown */
#define ETOOMANYREFS	(25+ETCP_EBASE)	/* Too many references: can't splice */
#define ETIMEDOUT	(26+ETCP_EBASE)	/* Connection timed out */
#define ECONNREFUSED	(27+ETCP_EBASE)	/* Connection refused */

/* random errors */
#define EHOSTDOWN	(28+ETCP_EBASE)	/* Host is down */
#define EHOSTUNREACH	(29+ETCP_EBASE)	/* No route to host */
#define	ETCP_ELIMIT	EHOSTUNREACH

/*
 * New pathname-handling error numbers.
 */
#define	ENAMI_EBASE	ETCP_ELIMIT
#define ELOOP		(1+ENAMI_EBASE)	/* Too many levels of symbolic links */
#define	ENAMETOOLONG	(2+ENAMI_EBASE)	/* File name too long */
#define	ENAMI_ELIMIT	ENAMETOOLONG

/*
 * Network filesystem error numbers (nfs).
 */
#define	ENFS_EBASE	ENAMI_ELIMIT
#define	ENOTEMPTY	(1+ENFS_EBASE)	/* Directory not empty */
#define	EDQUOT		(2+ENFS_EBASE)	/* Disc quota exceeded */
#define ESTALE		(3+ENFS_EBASE)	/* Stale NFS file handle */
#define	ENFSREMOTE	(4+ENFS_EBASE)	/* Too many levels of remote in path" */
#define	ENFS_ELIMIT	ENFSREMOTE

#define	ELASTERRNO	ENFS_ELIMIT

#if (defined(__EXTENSIONS__) && !defined(_POSIX_SOURCE))
#define TCP_EBASE 	ETCP_EBASE
#define	TCP_ELIMIT	ETCP_ELIMIT
#define	NAMI_EBASE	ENAMI_EBASE
#define	NAMI_ELIMIT	ENAMI_ELIMIT
#define	NFS_EBASE	ENFS_EBASE
#define	NFS_ELIMIT	ENFS_ELIMIT
#define	LASTERRNO	ELASTERRNO
#endif /* __EXTENSIONS__ && !_POSIX_SOURCE */

#endif /* !__SYS_ERRNO_H__ */
