/*      @(#)rnode.h	1.4 88/07/15 NFSSRC4.0 from 1.21 88/02/08 SMI      */
/*	Copyright (C) 1988, Sun Microsystems Inc. */

#ifndef __NFS_RNODE_H__
#define __NFS_RNODE_H__

/*
 * Remote file information structure.
 * The rnode is the "inode" for remote files.  It contains all the
 * information necessary to handle remote file on the client side.
 */
struct rnode {
	struct rnode	*r_next;	/* active rnode list */
	struct rnode	*r_prev;	/* back-pointer in active list */
	struct inode	*r_ip;		/* back-pointer to inode */
	struct inode	*r_specip;	/* special inode if device or fifo */
	fhandle_t	r_fh;		/* file handle */
	u_short		r_flags;	/* flags, see below */
	short		r_error;	/* async write error */
	daddr_t		r_lastr;	/* last block read (read-ahead) */
	lock_t		r_flaglock;	/* to protect r_flags */
	struct ucred	*r_cred;	/* current credentials */
	union {
	    struct {
		struct ucred	*ruu_unlcred;	/* unlinked credentials */
		char		*ruu_unlname;	/* unlinked regular file name */
		struct vnode	*ruu_unldvp;	/* unlinked file's parent dir */
	    } ru_unl;
	    struct {
		time_t		rus_symtime;	/* symlink expiration time */
		u_int		rus_symlen;	/* symlink value length */
		char		*rus_symval;	/* cached symlink contents */
	    } ru_sym;
	} r_u;
	struct nfsfattr	r_nfsattr;	/* cached nfs attributes */
	struct timeval	r_nfsattrtime;	/* time attributes become invalid */
	ino_t		r_number;	/* rnode number */
	u_short		r_refcount;	/* reference count from all inodes */
	u_short		r_iocount;	/* io operations in progress */
	sema_t		r_iowait;	/* io synchronization semaphore */
	sema_t		r_lock;		/* semaphore guarding attributes */
};

#define	r_unlcred	r_u.ru_unl.ruu_unlcred
#define	r_unlname	r_u.ru_unl.ruu_unlname
#define	r_unldvp	r_u.ru_unl.ruu_unldvp
#define	r_symval	r_u.ru_sym.rus_symval
#define	r_symlen	r_u.ru_sym.rus_symlen
#define	r_symtime	r_u.ru_sym.rus_symtime

/*
 * Flags
 */
#define	RLOCKED		0x0001		/* rnode is in use */
#define	RWANT		0x0002		/* someone wants a wakeup */
#define	RATTRVALID	0x0004		/* Attributes in the rnode are valid */
#define	REOF		0x0008		/* EOF encountered on read */
#define	RDIRTY		0x0010		/* dirty buffers may be in buf cache */
#define	RNOCACHE	0x0020		/* don't cache read and write blocks */
#define	RIOWAIT		0x0040		/* rnode waiting for io completion */
#define	RINODEWAIT	0x0100		/* waiting for iget in makenfsnode */

/*
 * Convert between vnode and rnode
 */
#define	rtov(rp)	((rp)->r_ip)
#define	vtor(vp)	((struct rnode *)((vp)->v_data))
#define	vtofh(vp)	(&(vtor(vp)->r_fh))
#define	rtofh(rp)	(&(rp)->r_fh)

#ifdef _KERNEL
#define	vtorfsid(vp)	(vtor(vp)->r_nfsattr.na_fsid)
#define	vtornodeid(vp)	(vtor(vp)->r_nfsattr.na_nodeid)
#define	rtorn(rp)	((rp)->r_number)

enum rget_how { RGET_DONTLOCK, RGET_LOCKRNODE };

void		rlock2(struct rnode *, struct rnode *);
void		initrnodes(u_int);
struct rnode	*rget(ino_t, enum rget_how);
struct rnode	*rsetinode(struct inode *);
void		rnewversion(struct inode *);
void		rdrop(struct rnode *, struct mntinfo *);
int		makenfsnode(fhandle_t *, struct nfsfattr *, struct vfs *,
			    struct vnode **);

/*
 * NFS inodes are numbered by their rnode's number and its version, so that
 * stale name cache hits can be detected.
 */
#define	getversion(inumber)	((inumber) >> 16)
#define	getrnumber(inumber)	((inumber) & 0xffff)
#define	makeinumber(vers, rnum)	(((vers) << 16) | (rnum))

/*
 * Lock and unlock rnodes.
 */
#define	rnode_is_locked(rp)	(valusema(&(rp)->r_lock) <= 0)

#define	rlock(rp)		psema(&(rp)->r_lock, PINOD)
#define	runlock(rp)		vsema(&(rp)->r_lock)

/*
 * Set and clear rnode flags.
 */
#define	rsetflag(rp, f)		{ int _s = splockspl((rp)->r_flaglock, splhi); \
				  (rp)->r_flags |= (f); \
				  spunlockspl((rp)->r_flaglock, _s); }
#define	rclrflag(rp, f)		{ int _s = splockspl((rp)->r_flaglock, splhi); \
				  (rp)->r_flags &= ~(f); \
				  spunlockspl((rp)->r_flaglock, _s); }

#define	nfsattr_inval(vp)	(vtor(vp)->r_nfsattrtime.tv_sec = 0)

#endif /* _KERNEL */
#endif /* __NFS_RNODE_H__ */
