#ifndef __SYS_GR1_DEV_H__
#define __SYS_GR1_DEV_H__

/**************************************************************************
 *									  *
 * 		 Copyright (C) 1986, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

/*
 * sys/gr1_dev.h
 *
 * $Revision: 1.40 $
 */

#undef GR1_DEBUG		/* Define to turn on debugging */


/******************************************************************************
 * General GR1 output driver defines
 *****************************************************************************/

#define NOFORCE		0	/* Do notforce an action */
#define FORCE		1	/* Force an action */

#ifdef	__GL_GLIMPORTS_H__
Error: do not include this file in the libgl source!!
#endif

#ifdef	GR1_AUXSCRN

#define MAXGFXBDS	2	/* Since we can't correctly probe a GR1 or 
				 * a VGR2, put VECTOR statements for all
				 * possible boards in master.d/system.gen
				 * and use this to control the number
				 * actually used by the kernel (if could
				 * probe correctly, could put MAXGFXBDS (and
				 * gr1_gbitab[]) master.d/gr1_dev and it would
				 * be a lbootable parameter */

/* 
 * Sanity checks
 */
#ifdef VGR

#if	MAXGFXBDS > 2
#undef	MAXGFXBDS
#define	MAXGFXBDS	2	/* XXX temp restriction; to have > 2, will
				 * need to:
				 *  - fix the code in sgaux_setinscrn()
				 *    to block the calling process under
				 *    certain conditions (see comment there)
				 *  - make sure that gbi_bd0cxid and is 
				 *    shadowed correctly when a process 
				 *    owns more than one auxiliary screen
				 */
#endif	/* MAXGFXBDS > 2 */

#else	/* !VGR */

#undef	MAXGFXBDS
#define	MAXGFXBDS	1

#endif	/* !VGR */


#define GR1_WMCONTEXT	16	/* Contexts used by the window manager */
#define GR1_MAXCONTEXT	(GR1_WMCONTEXT + MAXGFXBDS - 1)
				/* Max. number of kernel maintained contexts */
#define CXIDTOGBD(cxid)	(((cxid) < GR1_WMCONTEXT) ? 0 \
						  : ((cxid)-GR1_WMCONTEXT+1))
				/* convert cxid to the graphics board number 
				 * that it belongs to */
#define CXIDTOBIT(cxid)	(((cxid) < GR1_WMCONTEXT) ? (1 << (cxid)) : 1)
				/* convert cxid to a bit position for the bit
				 * masks in gr1_retrstat_t */
#define GBDTOCXID(bdnr)	((bdnr) ? (bdnr)+GR1_WMCONTEXT-1 : 0)
				/* convert a graphics board number into the
				 * first element of gr1_context[] for the
				 * board */

#else	/* !GR1_AUXSCRN */

#define	MAXGFXBDS		1
#define GR1_WMCONTEXT	16
#define GR1_MAXCONTEXT	GR1_WMCONTEXT
#define CXIDTOGBD(cxid)	0
#define CXIDTOBIT(cxid)	(1 << (cxid))
#define GBDTOCX(bdnr)	0

#endif	/* !GR1_AUXSCRN */

/*
 * The max length of user DMA transfer (might be broken into several DMA
 * transfers in kernel) is entire screen's worth of pixel data.
 */
#define GR1_MAXUDMA	(1280 * 1024 * sizeof(long))

/* Microcode word most sig 8 bits bit definitions */
/*XXX Maybe put these in some other header file? */
#define CX_INCMEM	0x08
#define CX_INCRE	0x20
#define CX_FIELD2	0x10
#define CX_DBSRC	0xC0
#define CX_RDFIFO	0x40


/* GR1 rev. A board defines */
#define REVA_EADDRLO	0x69010000	/* Lowest rev. A board ethernet addr */
#define REVA_EADDRHI	0x6901001B	/* Highest rev. A board ethernet addr */

#define SPLRETR()	spl7()
#define SPLGR1()	spl5()

#define K2_TO_PHYS(x)	(ctob(kvtokptbl(x)->pgm.pg_pfn) + poff(x))

#ifndef MIN
#define MIN(x, y)	(((x) < (y)) ? (x) : (y))
#endif


/******************************************************************************
 * Vertical Retrace structures and defines
 *****************************************************************************/

#define RETR_MAXMAPCOL		64 /* Max number col map entries can map during
				      one retrace */

#define RETR_NSUBCOLMAP 	XMAP_NCOLMAPENT / RETR_MAXMAPCOL
				/* Number sub col maps */

#define RETR_MAXBLINKS	 	20 /* Max number blinking color map entries*/

/*
 * Indices into oplist. These are the operations performed during retrace.
 * These indices are also used to determine the corresponding bit flag for
 * the operation in opflags (see OTOF()).
 *
 * NOTE: The order is very important because it represents the order in which
 * operations get executed during retrace.  The smaller the number, the
 * earlier the operation gets performed ("the higher the priority").
 */
#define N_OP		12		/* Total number retrace operations */
#define OP_HEADER	N_OP		/* Header of list (actually located at
					   end of list) */
#define OP_SWAPBUFFERS	0		/* Swap output buffers (double buf) */
#define OP_XMAPMODELO	1		/* Write an XMAP2 mode reg lo entry */
#define OP_XMAPMODEHI	2		/* Write an XMAP2 mode reg hi entry */
#define OP_CYCLEMAP	3		/* Blink color map entries */
#define OP_LASTDOALWAYS 3		/* Operations 0..OP_LASTDOALWAYS are
					 * always done for all boards each
					 * retrace */
#define OP_MAPCOLOR	4		/* Write color map entries */
#define OP_MAPAUXCOLOR	5		/* Write aux. color map entries */
#define OP_BLINK	6		/* Blink color map entries */
#define OP_SETCURSOR	7		/* Set cursor (glyph, type, origin) */
#define OP_MAPCURSCOLOR	8		/* Map cursor color */
#define OP_BLANKSCREEN	9		/* Blankscreen */
#define OP_GAMMARAMP	10		/* Write gamma ramp */
#define OP_LASTDURINGRETRACE 10		/* Operations beyond here don't need
					 * to be done during the actual
					 * retrace */
#define OP_GSYNC	11

#define OTOF(op)	(1 << op)	/* Converts operation indices to their
					   flag counterparts */

typedef struct optab {
	void (*func)(long);	/* Function that performs this operation */
	long cost;		/* Time to do operation, in arbitrary units */
} optab_t;

typedef struct oplist {
	int next;		/* Next operation to do in linked list */
	void (*func)(long);	/* Operation to do */
	long bdnr;		/* Do it for this board number */
} oplist_t;

/* Blink struct */
typedef struct gr1_blink {
	struct gr1_blink *last;	/* Double linked list of blink structs. */
	struct gr1_blink *next;
	struct gr1_blink *nownext; /* Linked list of blinks todo at next retr */
	long pid;		/* Proc. that owns this blink struct */
	unsigned long nextblink; /* Retrace count value to blink on */
	short rate;		/* How often to blink (in terms of retr.s) */
	unsigned short index;	/* Color table entry to blink */
	unsigned char red[2], green[2], blue[2]; /* Original and blink color */
	char toggle;		/* Index into color arrays. Toggles: 0 & 1 */
} gr1_blink_t;

/* Vertical retrace status structure */
typedef struct gr1_retrstat {
	unsigned long flags;		/* State flags */
	unsigned long opflags;		/* Flags indicating which operations
					  need to be done during next retrace */
	unsigned long retrcount;	/* Vertical retrace reference count.
					   Used for swapintervals */
	unsigned long retrwakeup;	/* Contexts to wakeup this retrace */
	unsigned long swapnow;		/* Contexts to swapbufs this retrace */
	unsigned long gsyncflags;	/* Contexts waiting for retrace */
	unsigned long enetaddr;		/* Ethernet addr. used to distinguish
					   rev. A vs. rev. B GR1 boards */

	/* These values change with different monitors */
	long xmaxscreen;		/* Max screen coordinate in x */
	long ymaxscreen;		/* Max screen coordinate in y */

	struct gr1_context *swapbufhead; /* Contexts to swap buf during retr */
	struct gr1_context *xmapmodelohead; /* Linked list of contexts to change
					       XMAP mode lo reg at next retr */
	struct gr1_context *xmapmodehihead; /* Linked list of contexts to change
					       XMAP mode hi reg at next retr */

	unsigned char dreg[NDREG];	/* Shadowed display reg's */

	struct mapcolor {
		short count;
		short submap;
		struct submaps {
			char empty;
			short lower;
			short upper;
		} submaps[RETR_NSUBCOLMAP];

		/* Color map shadow */
		unsigned char red[XMAP_NCOLMAPENT];
		unsigned char green[XMAP_NCOLMAPENT];
		unsigned char blue[XMAP_NCOLMAPENT];
	} mapcolor;

	struct mapauxcolor {
		/* Auxiliary color map shadow */
		unsigned char red[XMAP_NAUXMAPENT];
		unsigned char green[XMAP_NAUXMAPENT];
		unsigned char blue[XMAP_NAUXMAPENT];
	} mapauxcolor;

	struct gammaramp {
		unsigned short *red;   /* Pointer to red gamma ramp */
		unsigned short *green; /* Pointer to green gamma ramp */
		unsigned short *blue;  /* Pointer to blue gamma ramp */
	} gammaramp;

	struct cursor {
		/* These values may change with different monitors */
		long xmagic;	/* Magic x offset to add with this hardware */
		long ymagic;	/* Magic y offset to add with this hardware */

		long xoffset;   /* Offsets to use to get correct cursor x,y */
		long yoffset;	/* positions. Saved so don't do mult */
				/* additions every time move cursor */
		long xdelta;	/* When changing cursors save the change */
		long ydelta;	/* so that cursor will not jump */
		long xorigin;   /* Block cursor x origin within glyph */
		long yorigin;	/* Block cursor y origin within glyph */
		char type;
		unsigned char red;
		unsigned char green;
		unsigned char blue;
		unsigned char glyph[CURS_GLYPH_SIZE];
		char on;	/* Non-zero when cursor is on */
		char _dummy[3];
	} cursor;

	struct blink {
		gr1_blink_t *head;
		gr1_blink_t *tail;
		gr1_blink_t *nowhead;
		long count;
	} blink;

	struct gr1_context *cyclemaphead; /* Linked list of contexts to cycle
					     at next retr */
} gr1_retrstat_t;

/*
 * Values for gr1_retrstat_t.flags
 */
#define R_USERBLANK	0x00000001	/* User blanked screen */
#define R_TIMEBLANK	0x00000002	/* Timeout w/o input blanked screen */

#define R_LOSPL_INTR	0x00000004	/* replaces gr1_lospl_intr global */


/******************************************************************************
 * GR1 structures and defines
 *****************************************************************************/

/*
 * XXX If change sggr1_dma() or any of these defines, check the others!
 * Defines used in sggr1_dma()
 */
#define DMAMAXXLEN 	1280	/* Max x length for pixel DMA */
#define DMAMAXYLEN 	1024	/* Max y length for pixel DMA */
#define PGSPERSHOT	32		    /* Number pages map/lock at a shot*/
#define MAXPERSHOT	(NBPC * PGSPERSHOT) /* Max bytes map/lock at a shot */
#define MAXLNBYLN	(DMAMAXXLEN * 4)    /* Max DMA length for a line in
					       line by line DMA */
			/* DMA desc. array size worst case:
			 * (# lines on screen + # pg boudaries can cross)
			 * We do 1 chunk at a time double buffered so just need
			 * enough space for largest chunk. Must take into
			 * account # lines since line by line DMA requires at
			 * least 1 entry per line. */
#define DMADASIZE	(DMAMAXYLEN + PGSPERSHOT + 1)

/* Status structure for GR1 */
typedef struct gr1_hoststat {
	long cxid;		/* Context ID for active context */
	long wid;		/* Window ID for active context */
	long ncontext;		/* Number kernel supported graphics contexts */
	long nwid;		/* Number hardware window ID's */
	unsigned long flags;	/* Bit flags (defined below) */

	sema_t cxwait;		/* Queue processes waiting for a context */

	sggr1_gethwconfig_t hwconfig;	/* Hardware configuration */

	/*
	 * DMA descriptor array for context switch, pick/select, R/W pixels.
	 * Double bufferred.
	 */
	struct dma {
		sema_t lock;		/* Semaphore for locking gfx DMA */
		sema_t wait;		/* Semaphore for waiting for gfx DMA */

		gdmada_t da0[DMADASIZE];
		gdmada_t da1[DMADASIZE];
	} dma;
} gr1_hoststat_t;

/*
 * Values for gr1_hoststat_t.flags
 */
#define GR1_INITIALIZED	0x00000001	/* board initialized */

#ifdef	GR1_AUXSCRN
#define GR1AUX_WRITING  0x80000000	/* this auxiliary screen is currently 
					 * being written to, i.e. a 
					 * SGAUX_SETOUTSCRN  has switched 
					 * graphics output to it; never set 
					 * for board 0 */
#define GR1AUX_TRACKING 0x40000000	/* this auxiliary screen is currently 
					 * being used for tracking to, i.e. a 
					 * SGAUX_SETINSCRN has switched 
					 * graphics input to it; never set for 
					 * board 0 */
#endif	/* GR1_AUXSCRN */


/******************************************************************************
 * Data structure corresponding to a graphics context on the GR1 board
 *****************************************************************************/

typedef struct gr1_mapbuf {
	caddr_t ubufp;		/* user virtual address of buffer */
	long buflen;		/* size in bytes of user's buffer */
	caddr_t kbufp;		/* kernel virtual address of user's buffer */
} gr1_mapbuf_t;


/* XMAP mode reg struct (for retrace module) */
typedef struct gr1_xmapmode {
	struct gr1_context *next; /* Linked list of contexts to change
				     mode reg at next retr */
	unsigned char mode;	  /* XMAP2 mode reg shadow */
} gr1_xmapmode_t;

/* Cyclemap struct (for retrace module) */
typedef struct gr1_cyclemap {
	struct gr1_context *next; /* Linked list of contexts to cycle at next
				     retr */
	unsigned long nextcycle;  /* Retrace count value to do next cycle */
	short curmap;		  /* Current map being used */
	struct maps {
		short nextmap;	  /* Next map to cycle to */
		short duration;	  /* How long use this map in terms of retr.s */
	} maps[XMAP_NMULTIMAP];
} gr1_cyclemap_t;

/* Context status structure */
typedef struct gr1_context {
	long wid;		/* Window ID for this context */
	unsigned long flags;	/* bit flags */
	long size;		/* Graphics saved context size */

	unsigned long swapinterval; /* Min. retraces between swapbuffers */
	unsigned long nextswap;	/* Retrace count value to swapbuffers at */

	int cx_feedcount;	/* feedback characters copied to user's buf */
	int cx_waiters;		/* counter of suspended procs */

	long finish0;		/* Saved FINISH0 flag from context switch */
	char *gfxcxp;		/* Graphics saved context pointer */
	struct graph_s *graphp;	/* Pointer to proc associated w/ this context */

	sema_t retr_wait;	/* semaphore for procs waiting for retrace */
	sema_t cx_wait;		/* semaphore to suspend procs */

	gr1_mapbuf_t dmabuf;	/* dual mapped buffer for dma */
	gr1_mapbuf_t pickbuf;	/* dual mapped buffer for picking/feedback */
	gr1_mapbuf_t wmbuf;	/* dual mapped buffer for WM cx load/store */

	/* Struct.s that retrace module uses */
	gr1_xmapmode_t xmapmodelo;	/* XMAP mode lo reg struct */
	gr1_xmapmode_t xmapmodehi;	/* XMAP mode hi reg struct */
	gr1_cyclemap_t cyclemap;	/* Cyclemap struct */
} gr1_context_t;

/*
 * Values for gr1_context_t.flags
 */
#define	GR1_CX_DIRTY		0x0001	/* Has not been cleaned up */
#define	GR1_CX_USED		0x0002	/* Not new context: has saved state */
#define	GR1_CX_NOGECX		0x0004	/* Space for GE context was freed */
#define	GR1_CX_XMAPMODELO	0x0008	/* Change XMAP mode lo reg at retr */
#define	GR1_CX_XMAPMODEHI	0x0010	/* Change XMAP mode hi reg at retr */
#define	GR1_CX_SWAP_PENDING	0x0020	/* swap buffers pending */
#define	GR1_CX_GSYNC		0x0040	/* Gsync pending */
#define	GR1_CX_DMA		0x0080	/* Doing user graphics DMA */
#define	GR1_CX_PICKING		0x0100	/* pick/select in progress */
#define	GR1_CX_ENDPICK		0x0200	/* endpick/select in progress */
#define	GR1_CX_OVERFLOW		0x0400	/* pick/select buffer too small */
#define	GR1_CX_CYCLEMAP		0x0800	/* Cyclemap this context at retrace */
#ifdef  GR1_AUXSCRN
#define GR1_CX_BD0CXSTOLEN	0x1000	/* WM has stolen the board 0 context
					 * of the process that owns this
					 * (non-board-0) context */
#endif /* GR1_AUXSCRN */

#define VGR1_VR_SLOT	0
#define VGR1_FIFO_SLOT	1
#define	VGR1_GE_SLOT	2


typedef struct {	    /* Table of interrupt handlers */
	int (*vec)();
	int arg;
} vect_t;

#ifdef	GR1_AUXSCRN

/* per board data */
struct gr1_board_info {
    paddr_t gbi_addr;	    /* base address in K1 space of this board (values
			     * for GRP); if 0, then board is missing or not
			     * functioning; filled in by gr1_edtinit() */
    struct edt *gbi_edtp;   /* pointer to edt entry for this board; need
			     * to have this because sometimes we need
			     * know about board addresses and vectors before
			     * gr1_edtinit() is called by the kernel */
    gr1_hoststat_t  gbi_hoststat;
    gr1_retrstat_t  gbi_retrstat;
    short gbi_pid;	    /* process that owns this board; zero if
			     * none or board 0 (which doesn't have an
			     * owner) */
    short gbi_cursorx;	    /* actual cursor x */
    short gbi_cursory;	    /* actual cursor y */
    char gbi_finish0;	    /* Must save FINISH0 flag and write it into GE
			     * saved context after DMA has completed.  The GE
			     * can write the flag but it cannot read it so host
			     * reads it and puts it in the saved context for GE
			     * to restore.  This is done in separate routines
			     * that are called at different times so we use
			     * some global variables.
			     */
    signed char gbi_bd0cxid;/* id of the owner of this board's board 0 context;
			     * XXX for now, the code doesn't support a process
			     * owning more than one auxiliary screen
			     */
#if 0	/* don't need this for auxscrn implementation */
    long gbi_nr;	    /* board number = index of this array simplifies
			     * determining board number when access is by
			     * address of element of array; board number is a
			     * minor of a device, hence can only be one byte
			     */
#endif
    long gbi_boardtype;	    /* type of board detected at this slot */
};

#define gr1_hoststat	gr1_gbitab[bdnr].gbi_hoststat
#define gr1_retrstat	gr1_gbitab[bdnr].gbi_retrstat
#define gr1_finish0	gr1_gbitab[bdnr].gbi_finish0
#define gr1_boardtype	gr1_gbitab[bdnr].gbi_boardtype

#define GR1_AUXSCRN_ASSERT	ASSERT

/* use in for loops to skip over missing boards */
#define	SkipMissing(bdnr)	if (gr1_gbitab[bdnr].gbi_addr == 0) continue

extern struct gr1_board_info gr1_gbitab[]; 

#else	/* !GR1_AUXSCRN */

extern gr1_hoststat_t gr1_hoststat;
extern gr1_retrstat_t gr1_retrstat;
extern long gr1_boardtype;

#define GR1_AUXSCRN_ASSERT(x)
#define	SkipMissing(bdnr)

#endif	/* !GR1_AUXSCRN */

extern gr1_context_t gr1_context[];
extern long gr1_lospl_intr;			/*XXX-- TJN */
extern long gr1_retrwakeup;
extern long gr1_skipretr;

#ifdef GR1_DEBUG
/* Set these variables in kernel debugger to get more info */
extern long GR1_DEBUG_CX;	/* Graphics context switching */
#endif /* GR1_DEBUG */

#endif /* __SYS_GR1_DEV_H__ */
