#ifndef __SYS_GR2_IF_H_
#define __SYS_GR2_IF_H_
/* $Header $ */

/*********************************************************************
*
*	Header file shared between graphics and kernel.
*
*********************************************************************/
#define CONSTMEM        0       /* constants memory  base:     0 size:  256 */
/* State of contexts */
#define	TYPE_NEW		0
#define TYPE_GE			1	
#define	TYPE_HOST		2
#define TYPE_DUMP		3
#define TYPE_NOT_FOUND		4

/* Kinds of contexts */
#define CTX_SMALL		0
#define CTX_LIGHT		1
#define CTX_X			2

/* Bitmask for data buffer types */
#define DATA_SCRATCH		0x1
#define DATA_NURBS		0x2
#define DATA_SPOTLIGHT		0x4
#define DATA_VOLUME		0x8
#define DATA_VOLUME8K		0x10
#define DATA_VOLUME16K		0x20

/* Pipe values of data buffer types */
#define PIPE_SCRATCH            0
#define PIPE_NURBS              1
#define PIPE_SPOTLIGHT          2
#define PIPE_VOLUME             3
#define PIPE_VOLUME8K           4
#define PIPE_VOLUME16K          5

/* Size of data storage areas */
#define SIZE_NURBS              1200
#define SIZE_SPOTLIGHT          256
#define SIZE_VOLUME             256
#define SIZE_VOLUME8K           8192
#define SIZE_VOLUME16K          16384


#define GE_KDATA                 0x0
#define GE_RESET                0

/* Shared RAM location */
#define	GR2_INTERRUPT_TYPE	768
#define GR2_ERRNO		769
#define CX_SAVE_SIZE		770
#define CX_SAVE_ID		771
#define CX_SCRATCH_SIZE		772
#define CX_SCRATCH_ID		773	

/* Shared RAM location to report RE version */
#define READ_RE_REGS		32760
#define RE3_VERSION		READ_RE_REGS

/* Shared RAM location and return data to check whether the textport microcode
 * is running still. */
#define TP_PROBE_ID		32767
#define UCODE_TP		1
#define UCODE_GLX		2
#define UCODE_DIAG		4

/*------------------------------------------------------*/
/* 		CONTEXT SWITCHING TOKENS (480-511)	*/
/*							*/
/* 480-495 are normal tokens only read at a fetch  	*/
/* 496-511 (1f0-0x1ff) are special HQ2 tokens that 	*/
/* interrupt HQ2 microcode execution and save HQ2 PC	*/
/*------------------------------------------------------*/

/* Tokens that do not execute HQ2 context switch -- start at 480 */
#define	GE_RESTORE_CTX		480
#define	GE_SAVE_SHRAM		481
#define	GE_RESTORE_SHRAM	482
#define GE_SAVE_DATA		483
#define GE_RESTORE_DATA		484
#define GE_VALIDATE_CLIP	485
#define	GE_RESTORE_X_CTX	486

/* Tokens to initialize context */
#define GE_INIT		1	

/* Tokens that execute HQ2 context switch -- 496 to 511 */
#define GE_CTX0			496	/* needed for simulator */
#define GE_CTX7			503	/* needed for simulator */
#define	GE_SAVE_CTX 		GE_CTX0
#define	GE_SAVE_X_CTX 		497

/* Token to write to RE register, pass RE address and data as arg */
#define	GE_LOADRE		2
#define RE_TOPSCAN_ADDR		108	

/* Types of GE interrupts */
#define INT_INIT	1
#define INT_ERROR	2

/*XXX Error number of INT_ERROR */
/*XXX Added later !! */ 

/*
** XXX--standalone version of gecmds.h
*/
#define PUC_DATA                0
#define PUC_INIT                401
#define PUC_COLOR               402
#define PUC_FINISH              403     /* ??? */
#define PUC_PNT2I               404
#define PUC_RECTI2D             405
#define PUC_CMOV2I              406
#define PUC_LINE2I              407
#define PUC_DRAWCHAR            408
#define PUC_RECTCOPY		409

#define DIAG_DATA		0
#define DIAG_REDMA		417
#define PASS32			418
#define DIAG_GESHRAM		419

#define GE7GLOBALS

/* VC1 cursor mode - normal pattern, crosshair*/
#define GR2_CURS_NORMAL	0
#define GR2_CURS_CROSS 3

/* VC1 SRAM Memory Layout */
#define GR2_VIDTIM_LST_BASE 	    0
#define GR2_VIDTIM_CURSLST_BASE     0x400
#define GR2_VIDTIM_FRMT_BASE        0x800
#define GR2_VIDTIM_CURSFRMT_BASE    0x900 /* Frametable definitely < 200 bytes*/
#define GR2_INTERLACED_MON	    0x9F0 /* Is the monitor interlaced?*/
#define GR2_SCREENWIDTH		    0x9F2 /* Xserver needs this info for
					   * calculating width of DID table
					   */
#define GR2_NEXTDID_ADDR	    0x9F4 /*Next available location in VC1 sram*/
#define GR2_CURSOR0_BASE            0xA00 /* Each 32X32 bitmap = 0x100 */
#define GR2_DID_FRMT_BASE           0xB00
#define GR2_DID_MAX_FMTSIZE         0x900 /* Max # of line in 
					   * frametable = 2 * 1024
				           */
				/*0x1300*/
#define GR2_DID_LST_BASE    	GR2_DID_FRMT_BASE + GR2_DID_MAX_FMTSIZE 

#define GR2_DID_LST_END    	0x8000

#define GR2_NTSC_HORIZ_RES	640
#define GR2_PAL_HORIZ_RES	780	
#define GR2_A343_HORIZ_RES	1280

/* Colormap Constants*/
/* cu_reg = 0
 * pg_reg = 14 
 */    
#define GR2_12BITCMAP_BASE	0	
#define GR2_8BITCMAP_BASE	4096
#define GR2_4BITCMAP_BASE	5120	
#define GR2_4BITMMCMAP_BASE	6144
#define GR2_CURSCMAP_BASE 	7168
#define GR2_AUXOU_BASE		7232		
#define GR2_4BITMMPIXOU_BASE	7296		
#define GR2_4BITPIXOU_BASE	7424		
#define GR2_8BITPIXOU_BASE	7680		

/* 
 * External XMAP fifostatus flags 
 */
#define GR2_FIFO_EMPTY          0x1
#define GR2_FIFO_HALF_FULL      0x2
#define GR2_FIFO_OVF            0x4


#endif /* __SYS_GR2_IF_H__ */
