/**************************************************************************
 *									  *
 * 		 Copyright (C) 1987, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

/*
 * $Revision: 1.34 $
 */

#ifndef _GTX_H_
#define _GTX_H_

#define GTX_FIFO_SIZE		0x4000	/* size of fifo hardware to map */

#if defined(IP17)
#define GTX_TRIGGER_SIZE	0x2000	/* size of trigger hardware to map */
#else
#define GTX_TRIGGER_SIZE	0x1000	/* size of trigger hardware to map */
#endif
#define GTX_FEED_SIZE		0x3000	/* size of feedback area to map */
#define GTX_MAP_SIZE		(GTX_FIFO_SIZE + GTX_TRIGGER_SIZE + GTX_FEED_SIZE)
#define GTX_TAG_SIZE		0x10000	/* size of tagram to map for diagnostics */
#define GT_DRAM_SIZE		0x100000/* size of GM1 DRAM for diagnostics */

#define GTX_WID_CNT		16		/* number of display contexts in hw */
#define GTX_PCX_CNT		16		/* number of pipe contexts in hw */
#define GTX_PCX_SIZE		(32*1024)	/* maximum size of pipe context */
#define GTX_POLL_TIMEOUT	1000000		/* spin gm busy timeout */
#define GTX_FIFO_TIMEOUT	500		/* spin fifo full timeout */
#define GTX_GM_TIMEOUT		HZ		/* timed DMA timeout */
#define SPLGTX			spl5

/*
 * Structure returned by ioctl GETBOARDINFO when executed on a GTX.
 */
struct gtx_info {
	struct gfx_info gfx_info; 	/* device independent information */

	/* summary of GTX specific information */
	int	rv_state;	/* display hardware information */
	int	flags;		/* flag indicating configuration of head */
};

#define GTX_BASE		10000			/* XXX really device dependent base defined elswhere */
#define GTX_RECTOP		(GTX_BASE + 1)
#define	GTX_CALLTOGM		(GTX_BASE + 2)
#define GTX_SETCURSORMODE	(GTX_BASE + 3)
#define GTX_SETCURSORCACHEID	(GTX_BASE + 4)
#define GTX_SETCURSORCOLORS	(GTX_BASE + 5)
#define GTX_RTNBUFBEGIN		(GTX_BASE + 6)
#define GTX_RTNBUFEND		(GTX_BASE + 7)
#define GTX_RCGREG		(GTX_BASE + 8)
#define GTX_WCGREG		(GTX_BASE + 9)
#define	GTX_PIXDMA		(GTX_BASE + 10)
#define GTX_SETDISPLAYMODE	(GTX_BASE + 11)

/* defines for the CG register */
#define CG_MODE_REG		0
#define CG_CONTROL_REG		1
#define CG_HORPHASE_REG		2
#define CG_SUBPHASE_REG		3
#define CG_LASTREG		CG_SUBPHASE_REG


/*
 *   GTX Ioctl args
 */

struct gtx_setobscured_args {
	int rnid;
	int obscuredflag;
};

struct gtx_setwinpos_args {
	int rnid;
	int xo,yo,xs,ys;
};

struct gtx_setwid_args {
	int rnid;
	int wid;
};

struct gtx_setdisplaymode_args {
   	long int wid;
	unsigned long int displaymode;
};

struct gtx_setcursormode_args {
	int	mode;
};

struct gtx_setcursorcacheid_args {
	int	cache_id;
};

struct gtx_setcursorcolors_args {
	int	colors[3];
};

struct gtx_rectop_args {
	int	flag;		/* 0 = load, 1 = store _to_ main memory */
	caddr_t	buffer;		/* user virtual address of buffer */
	int	pixsize;	/* size in bytes of each pixel */
	int	x;		/* x coord of lower left corner */
	int	y;		/* y coord of lower left corner */
	int	width;		/* width in pixels of rect before zoom */
	int	height;		/* height in pixels of rect before zoom */
	int	xzoom;		/* x zoom factor */
	int	yzoom;		/* y zoom factor */
};

struct gtx_calltogm_args {
	/*
	 * NOTE: sendp actually points to struct gm_generic_cmd 
	 *	 recvp actually points to struct gm_generic_resp.
	 */
	int	cmd;		/* command to send to GM		*/
	void	*sendp;		/* Pointer to packet to send to GM	*/
	long	slen;		/* length in bytes of the packet	*/
	void	*recvp;		/* Ptr to recv packet be stuffed by GM  */
	long	rlen;		/* length in byte of receive packet	*/
};

struct gtx_rtnbufbegin_args {
#define	GM_RN_PICKING		0x2	/* pick/select in progress */
#define	GM_RN_FEEDBACK		0x4	/* feedback operation in progress */
	long flags;		/* pick or feedback? */
        char *buffer;           /* user buffer virtual address */
        long buflen;            /* length of buffer */
};

struct gtx_rtnbufend_args {
	unsigned int gecmd;	/* GE cmd */
	unsigned int gedata;	/* GE data */
        long overflow;          /* buffer overflow flag */
};

struct gtx_cgreg_args {		/* GTX_RCGREG / GTX_WCGREG arguments */
	unsigned int reg;	/* register number */
	unsigned char value;	/* value */
};

struct gtx_pixdma_args {
	int	dir;		/* 0 = read, 1 = write			*/
	char	*buf;		/* pointer to src or dest buffer	*/
	int	cnt;		/* buffer length in bytes		*/
	int	off;		/* pixel buffer offset in bytes		*/
};

/* arbitrary size chosen so we don't lock down too much mem */
#define GTX_MAX_RTNBUF		(128 * NBPC)	/* Return buf max size (bytes)*/

#ifdef _KERNEL
/*
 * GM2 external status register (at base address provided in edtinit)
 *	status register is read only...
 *	to reset the board, write 1 then 0...
 */
#define	GM_SR_FIFO_NOT_HALF_FULL	0x01
#define	GM_SR_NOT_RESET			0x02
#define GM_SR_NOT_HALTED		0x04
#define	GM_CONTROLS_FIFO		0x08		/* GM3 only */
#define	GM_SR_DATAFIFO_NOT_HALF_FULL	0x10		/* GM3 only */
#define	GM_REVISION_MASK		0xe0		/* GM2, GM3 */
#define	GM_REVISION_2			0xa0		/* GM 2 */
#define	GM_REVISION_3			0x00		/* GM 3 */
#define GM_3WAYCXT_SHIFT		12
#define GM_3WAYCXT_MASK			(0x7 << GM_3WAYCXT_SHIFT)
#define GM_TAGADDR_SHIFT		8
#define GM_TAGADDR_MASK			(0xf << GM_TAGADDR_SHIFT)

/*
 * GM2 3way count restore register resides at this offset from
 * the external status register.  it is a write only 32 bit register
 */
#define GM_3WAYCXT_OFFSET		0x200000

/*
 * the following structures are utilized to reference real
 * hardware registers.
 */
struct gm2_hw_reg {
	unsigned int reg;
};
struct gm1_hw_reg {
	unsigned char reg;
};

/*
 * GM relative offsets
 */
#define GM_HTOGM_OFFSET		0x00000000
#define GM_GMTOH_OFFSET		0x00000800
#define GM_SHMEM_OFFSET		0x00001400

/*
 * GM1/2 specific offsets
 */
#define GM1_COMM_OFFSET		0x00000400
#define GM1_FEED_OFFSET		0x00002000
#define GM1_PCXAREA_OFFSET	0x00032000
#define GM2_PIPE_OFFSET		0x00300000
#define GM2_TAG_OFFSET		0x00100000

/*
 * the following structure is utilized to access the
 * graphics pipe from the host.
 */
struct gtx_pipe {
   union { float f; long i; } xf;
   union { float f; long i; } yf;
   union { float f; long i; } zf;
   union { float f; long i; } wf;
   long xd, yd, zd, wd;
   long xi, yi, zi, wi;
   long xyi;
   long lswd;
   long zwi;
   long xyzwi;
};

/* XXX should be called something else, flag bits in gfxp->gx_flags */
#define	GM_CX_PICKING		0x2	/* pick/select in progress */
#define	GM_CX_FEEDBACK		0x4	/* feedback operation in progress */

/*
 * each instance of a GTX graphics head requires one of the following
 * structures to control the head.
 */
struct gtx_data {
	struct gfx_data		gfx_data;	/* device independent data */
	struct gtx_info		info;		/* gtx specific info */
	struct rrm_rnode	*rrn;		/* current rendering node */
	int			gmtype;		/* flag indicating gm1/gm2 */
	char			*gm_pipebase;	/* K1 pipe base addr gm1/gm2*/
	char			*trigbase;	/* K1 3-way base addr for GM2 */
	host_to_gm_comm_t	*htogm;		/* host to gm comm area */
	gm_to_host_comm_t	*gmtoh;		/* gm to host area */
	lock_t			gmlock;		/* to lock the comm area */
	sema_t			gmcmd;		/* to semaphore the command interface */
	sema_t			gmdma;		/* to synchronise users doing GM DMA */
	sema_t			gmwaitdma;	/* semaphore to indicate DMA done */
	sema_t			gmwaitcmd;	/* a place to wait for GM response */
	sema_t			gmwaitabt;	/* a place to wait for GM aborts */
	sema_t			gmwaitcxt;	/* a place to wait for GM cxt switches */
	int			timiddma;	/* timeout ID for GM DMAs */
	int			timidcmd;	/* timeout ID for GM CMDs */
	int			timidabt;	/* timeout ID for GM ABORTs */
	int			timidcxt;	/* timeout ID for GM CXT switches */
	int	timeoutcnt;			/* Number of times fifo timeout */
	int	swapwid[GTX_WID_CNT];		/* WID map for swapbuffers */
	uint			pcx_pt[16];	/* Phys addr to pass to GM for DMA of heavyweight cxt switch */
	char			*pcx_area;	/* Pointer to non-cached, non-paged area for GM DMA of context */
	int			gmdmalen;	/* bytes completed for last GM dma */
	void			*recvp;		/* Kernel space to receive GM data */
	int			recvlen;	/* Size of kernel space to receive GM data */
	volatile int		gmready;	/* host_reset_complete is done (volatile since it is set by intr handler)*/
	int			feedp[16];	/* host copy of feedback pointers */
	int			pixflag;	/* pixel dma in progress */
	struct rrm_rnode *loadedpcxs[GTX_PCX_CNT];
	unsigned long loadedmodes[GTX_WID_CNT];
	struct rrm_rnode *swaprnp[GTX_WID_CNT];
	struct rrm_rnode *retracelist;		/* waiting for vertical retrace list */
	ushort			retraceid;	/* unique retrace id generator */
	short			must_deactivate;

	/* GM1 SPECIFIC FIELDS */
	unsigned char gm1vmevec;		/* saved VME vector value */
	volatile struct gm1_hw_reg *gm1status;	/* K1 status addr for GM1 */
	volatile struct gm1_hw_reg *gm1interrupt;/* KI interrupt addr for GM1 */
	volatile struct gm1_hw_reg *gm1disable;	/* K1 interrupt disable address */
	volatile struct gm1_hw_reg *gm1vector;	/* K1 interrupt vector address */
	char			*gm1drambase;	/* K1 dram base addr for GM1 */
	volatile gm1_shared_t	*gm1shmem;	/* shared gm state info for GM1 */

	/* GM2 SPECIFIC FIELDS */
	int			gm2slot;	/* slot ID for GM2 */
	volatile struct gm2_hw_reg *gm2status;	/* K1 status addr for GM2 */
	volatile struct gm2_hw_reg *gm23waycxt[16];/* K1 3waycxt addr for GM2 */
	char			*gm2tagbase;	/* K1 tag ram base addr for GM2 */
	volatile gm2_shared_t	*gm2shmem;	/* shared gm state info for GM2 */
};

/*
 * Device DEPENDENT rendering node structure
 */
struct gtx_rnode {
	unsigned long		gm3waycxt;
	struct RRM_ValidateClip validateclip;	/* squirreled-away data for loading during cxt sw (if rnid!=0) */

        /* user return buffer to copy out data at interrupt time */
	char			*rtnbufu;	/* user virtual address of buffer */
	char			*rtnbufk;	/* kernel virtual address of buffer */
	long			rtnbuflen;	/* buffer size (bytes) */
	long			rtnbufcount;	/* number bytes in buffer */
	long			rtnbufoverflow;	/* buffer overflowed (not a flag because we
						   change this at interrupt time, which could
						   get flags out of sync) */
	unsigned long		flags;
	struct rrm_rnode *retracenext;		/* waiting for vertical retrace next pointer */
	ushort			retraceid;	/* retrace id for this guy */
#ifdef IP17
	unsigned long 		ip17p3way_cntcmdreg;
	unsigned long		ip17p3way_ABCDregs[4];
#endif
};
#endif /* _KERNEL */

#endif /* _GTX_H_ */
