#ifndef __SYS_JAGTAPE_H__
#define __SYS_JAGTAPE_H__

/* supported tape types */
#define JGEXABYTE8200	1	/* Exabyte EXB-8200 8mm cartridge tape */
#define JGEXABYTE8500	2	/* Exabyte EXB-8200 8mm cartridge tape */
#define JGKENNEDY96X2	3	/* Kennedy 96[16]2 1/2" 9 track */

#define JAGTAPECTLR(dev) (emajor(dev) & 7)
#define JAGTAPEUNIT(dev) ((int) jag_unitnum[(dev & 0xF0) >> 4])

#define GETDENSE(dev)	(dev & 0x3)
#define IS_JGKENNEDY(uit)	((uit)->typspec->jg_type == JGKENNEDY96X2)
#define IS_JGEXA8500(uit)	((uit)->typspec->jg_type == JGEXABYTE8500)
#define IS_JGEXABYTE(uit)	(((uit)->typspec->jg_type == JGEXABYTE8500) || \
				 ((uit)->typspec->jg_type == JGEXABYTE8200))

#define MAX_INQ_VID	8	/* # bytes for vendor ID */
#define MAX_INQ_PID	16	/* # bytes for product ID */
#define MAX_DENSE	4	/* # of different densities supported */
#define INV_DENSE	0xff	/* invalid density value */

struct jag_types {
	u_char	jg_invtype;	/* type from tape types in invent.h */
	u_char	jg_type;	/* type from tape types in invent.h */
	u_char	jg_vlen;	/* length of vendor string */
	u_char	jg_plen;	/* length of vendor string */
	u_char	jg_vid[MAX_INQ_VID];	/* vendor ID */
	u_char	jg_pid[MAX_INQ_PID];	/* product ID */
	u_char	jg_dens[MAX_DENSE];	/* density values */
	ulong	jg_cap;		/* Capability list */
	ulong	jg_blksz;	/* default fixed blocksize */
	ulong	jg_recblksz;	/* recommended blocking factor (in bytes) */
};

typedef struct jag_types jag_types_t;

struct jagtapeinfo
{
	u_char	scsi[10];	/* buffer for scsi commands */
	char	justopened;	/* cleared on first tape move after open */
	u_char  speed;	/* current speed setting */
	u_char  density;	/* current density */
	uint	state;		/* state and directives for session */
	uint	openflags;	/* flags used at open time */
	int	minsize;
	int	maxsize;
	int	cursize;
	int 	fixsize;
	int	resid;
	jag_types_t *typspec;	/* tape specific info. */
};

/*
 * constants for jagtapeinfo.state
 */
#define TPS_ONLINE		0x00000001	/* ready for i/o */

#define TPS_BOT			0x00000002	/* at BOT */
#define TPS_EW			0x00000004	/* in early warning zone */
#define TPS_EOD			0x00000008	/* end of data (blank tape) */
#define TPS_EOM			0x00000010	/* end of media (no tape) */

#define TPS_BFM			0x00000020	/* at bottom of filemark */
#define TPS_TFM			0x00000040	/* at top of filemark */
#define TPS_HITFMSHORT		0x00000080	/* Short read into a filemark */
#define TPS_FM (TPS_BFM | TPS_TFM | TPS_HITFMSHORT)

#define TPS_READ		0x00000100	/* reading has been done */
#define TPS_WRITE		0x00000200	/* writing has been done */
#define TPS_RW (TPS_READ | TPS_WRITE)

#define TPS_WRP			0x02000000	/* tape is write protected */
#define TPS_OPEN		0x04000000	/* device open */
#define TPS_ANSI		0x08000000	/* allow write past EW */
#define TPS_SILI		0x10000000	/* suppress ILI */
#define TPS_REWIND		0x20000000	/* rewind on close */
#define TPS_VARIABLE		0x40000000	/* variable mode */
#define TPS_BUFFERED		0x80000000	/* buffered i/o */

/*
 * constants defined for certain SCSI commands
 */
#define TPLOAD_YES 1
#define TPLOAD_NO 2
#define TPLOAD_RETENSION 3

#define TPSPACE_BLOCK 		0
#define TPSPACE_FILEMARK 	1
#define TPSPACE_SFILEMARK 	2
#define TPSPACE_EOD 		3

extern jag_types_t jag_known[];
extern int jagtape_numtypes;

#endif /* __SYS_JAGTAPE_H__ */
