/**************************************************************************
 *									  *
 * 		 Copyright (C) 1986, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef __SYS_REGION_H__
#define __SYS_REGION_H__
#ident	"$Revision: 3.38 $"

#include "sys/immu.h"
#include "sys/sema.h"

/*
 * Per region descriptor.  One is allocated for
 * every active region in the system.
 */

typedef	struct	region	{
	sema_t	r_lock;		/* Lock semaphore */
	unchar	r_type;		/* type of region */
	unchar	r_flags;	/* Various flags defined below. */
	char	r_color;
#define NOCOLOR 8
	int	r_passed;	/* used during paging - passover count */
				/* N.B. must be int */
	ushort	r_refcnt; 	/* number of users pointing at region */
	ushort	r_noshrink;	/* # of requests not to allow shrink */
	short	r_listsz; 	/* Number of page tables allocated to r_list */
	short	r_pgoff;	/* offset in pages to first page in region */
				/* this is used for regions that grow down */
	int	r_pgsz;		/* size in pages */
	int	r_pghnd;	/* getpages index into region */
	int	r_nvalid;	/* number of valid pages in region */
				/* of region */
	pde_t	**r_list;	/* Pointer to a list of pointers to	*/
				/* page tables and dbd's.		*/
	int	r_maxfsize;	/* Max growable size of the file	*/
				/* from which this region is loaded.	*/
	int	r_fileoff;	/* File offset of this mapping.		*/
	struct inode   *r_iptr;	/* pointer to inode where blocks are */
	struct file    *r_file;	/* File struct for mapped files.*/
	struct region  *r_forw;	/* forward link */
	struct region  *r_back;	/* backward link */
} reg_t;

/*
 * Region flags
 */
#define	RG_NOFREE	0x01	/* Don't free region on last detach */
#define	RG_VSWAP	0x02	/* Region has allocated virtual swap space */
#define RG_ILOCKED	0x04	/* tosspages locked inode */
#define RG_SWPPD	0x08	/* (Some of) page tables swapped out */
#define	RG_NOSWP	0x10	/* Don't swap out page tables */
#define	RG_AUTOGROW	0x20	/* Pages become autogrow after ip shrinks */
#define	RG_PHYS		0x40	/* Region describes physical memory */
#define	RG_ISOLATE	0x80	/* Region is isolated for real-time, no aging */

/*
 * Region types
 */
#define	RT_UNUSED	0	/* Region not being used.	*/
#define	RT_PRIVATE	1	/* Private (non-shared) region. */
#define RT_STEXT	2	/* Shared text region */
#define	RT_MEM		3	/* Regular swappable region */
#define RT_MAPFILE	4	/* Mapped file region 		*/

/*	Each process has a number of pregions which describe the
 *	regions which are attached to the process.
 */

typedef struct pregion {
	struct region	*p_reg;		/* Pointer to the region.	*/
	caddr_t		p_regva;	/* Virtual address of region.	*/
	ushort		p_flags;	/* Flags.			*/
	unchar		p_type;		/* Type.			*/
	struct pregion  *p_forw;	/* Forward link.		*/
	struct lckmap	*p_lockmap;	/* Map of locked pages.		*/
	struct gfx_gfx  *p_gfxp;	/* Pointer to gfx_gfx if p_type is PT_GR */
} preg_t;

/*	Pregion flags.
 */

#define	PF_RDONLY	0x01		/* Read-only attach.		*/
#define	PF_RDWR		0x02		/* Read-write attach.		*/
#define	PF_EXEC		0x04		/* Executable attach.		*/
#define PF_PROT		0x0f		/* permissions bits.		*/
#define	PF_DUP		0x10		/* Dup on fork.			*/
#define	PF_XDUP		0x20		/* Allocated through xdup().	*/
#define PF_NOSHARE	0x40		/* Do not share on sproc	*/
#define PF_AUDITR	0x80		/* Read audit			*/
#define PF_AUDITW	0x100		/* Write audit			*/

/*	Pregion types.
 *
 *	Note: do not change these without checking core() first.
 */

#define	PT_UNUSED	0x00		/* Unused region.		*/
#define	PT_TEXT		0x01		/* Text region.			*/
#define	PT_DATA		0x02		/* Data region.			*/
#define	PT_STACK	0x03		/* Stack region.		*/
#define	PT_SHMEM	0x04		/* Shared memory region.	*/
#define	PT_MEM		0x05		/* Generic memory region.	*/
					/* Space was Double mapped memory.*/
#define	PT_LIBTXT	0x06		/* Shared library text region.	*/
#define	PT_LIBDAT	0x07		/* Shared library data region.	*/
#define PT_GR 		0x08		/* Graphics region 		*/
#define PT_MAPFILE	0x09		/* Memory mapped file region 	*/

/*	``Opaque handle'' on virtual space.
 */

typedef struct vhandl {
	struct pregion	*v_preg;	/* Pointer to the pregion.	*/
	caddr_t		v_addr;		/* Virtual address of region.	*/
} vhandl_t;

#define v_getpreg(VT)	(VT)->v_preg
#define v_getaddr(VT)	(VT)->v_addr
#define v_getlen(VT)	ctob((VT)->v_preg->p_reg->r_pgsz)
#define v_getinode(VT)	(VT)->v_preg->p_reg->r_iptr
#define v_gethandle(VT)	(unsigned)(VT)->v_preg->p_reg


/*	Each mapped file region has an mregion
 *	structure attached to the inode.
 */
typedef struct mregion {
	struct region	*m_reg;		/* Pointer to the region.*/
	struct mregion	*m_forw;	/* Forward link.	*/
} mreg_t;

/*	Some generic flags for region routines.
 */
#define RF_DUP		0x0001		/* dup region into copy-on-write */
#define RF_FORCE	0x0004		/* force a private region */
#define RF_NOFLUSH	0x0008		/* do not flush tlbs (detachreg) */

extern sema_t	rlistlock;		/* Lock for the region list.	*/
extern lock_t	preg_lock;		/* Lock for the pregion list.	*/

extern preg_t	pregfree;	/* List of free pregions */
extern reg_t	ractive;	/* List of active regions */
extern reg_t	rfree;		/* List of free regions */
extern reg_t	sysreg;		/* Region for dynamic system space */

/* region allocator */
reg_t *allocreg(struct inode *, short, short);

/* region free routine */
void freereg(reg_t *);

/* Attach region to process. */
preg_t *attachreg(reg_t *, struct user *, caddr_t, unchar, unchar);

/* Detach region from process. */
void detachreg(preg_t *, struct user *, int);

/* Duplicate region (fork). */
reg_t *dupreg(reg_t *, int, int);

/* Grow region. */
int growreg(preg_t *, struct user *, int, unchar);

/* Load region from file to be paged in. */
int mapreg(preg_t *, caddr_t, struct inode *, int, int);

/* Load region from file. */
int loadreg(preg_t *, caddr_t, struct inode *, int, int);

/* v_* routines for mapped driver support */
int v_mapphys(/* vhandl_t *, void *, int */);	/* XX need void * */
int v_mapreg(vhandl_t *, int, int);

/* Find pregion from virtual address. */
preg_t *findreg(struct proc *, caddr_t);

/* Find pregion from virtual address. */
preg_t *vtopreg(struct proc *, caddr_t);
preg_t *lookuppreg(struct proc *, caddr_t);

/* Find find first pregion within address range. */
preg_t *findfpreg(struct proc *, caddr_t, caddr_t);

/* Find pregion of given type. */
preg_t *findpreg(struct proc *, unchar);

/* Find apprpriate spot for a region within address space */
caddr_t allocaddr(int, int);

/* Returns size of specified region in pages */
unsigned getregionsize(reg_t *);

/* Returns current size of proces */
int getpsize(struct proc *);

/* fill in page frame numbers */
int vtop(struct proc *, caddr_t, int, int *, int);
int vtopv(struct proc *, caddr_t, int, int *, int, int, int);

/* segment table maintenance */
void initsegtbl(void);
void zapsegtbl(struct proc *);
void segswtch(struct proc *);
void segresume(struct proc *);

/* Change protection for region. */
void chgprot(preg_t *, unchar);

/* change text region to a PRIVATE writable version */
preg_t *xdup(struct proc *, preg_t *);
preg_t *getprivatespace(struct proc *, caddr_t);
int ispcwriteable(struct proc *, caddr_t, int);

/* Initialize the region table. */
void reginit(void);
void reg2init(void);
void vrelvm(void);

#define	reglock(RP)	psema(&(RP)->r_lock, PZERO)
#define	creglock(RP)	cpsema(&(RP)->r_lock)
#define	regrele(RP)	{ ASSERT(valusema(&(RP)->r_lock) <= 0); \
			vsema(&(RP)->r_lock); }

#define rlstlock()	{ appsema(&rlistlock, PZERO); }
#define rlstunlock()	{ APSERT(valusema(&rlistlock) <= 0); \
			apvsema(&rlistlock); }

#define	preglock(P)	psema(&(P)->p_preglock, PZERO)
#define	pregunlock(P)	{ ASSERT(valusema(&(P)->p_preglock) <= 0); \
			vsema(&(P)->p_preglock); }


#ifdef NOSWAPPGTBL

#define stopti(segno)	(segno)
#define segtopgtbl(rp, segno)	((pde_t *) (rp)->r_list[segno])
#define pgtopgtbl(rp, pgno)	((pde_t *) (rp)->r_list[(pgno)/NPGPT])
#define pgtopde(rp, pgno)	\
		((pde_t *)&((rp)->r_list[(pgno)/NPGPT][(pgno)%NPGPT]))

#else  /* SWAPPGTBL */

#define stopti(segno)	((segno) * sizeof(pde_t) / sizeof(pde_t *))
#define segtopgtbl(rp, segno)	((pde_t *) (rp)->r_list[stopti(segno)])
#define pgtopgtbl(rp, pgno)	((pde_t *) (rp)->r_list[stopti((pgno)/NPGPT)])
#define pgtopde(rp, pgno)	\
		((pde_t *)&((rp)->r_list[stopti((pgno)/NPGPT)][(pgno)%NPGPT]))
#endif /* SWAPPGTBL */

#endif /* __SYS_REGION_H__ */
