
/* 
 * this structure is used by GL and kernel to tell each other about
 * the use of linear address memory. 
 *
 * XXX - this file is used by both venice and vgx/vgx t texture managers even
 * though the two are slightly different. Perhaps it should be two files.
 */

/* these same ideas are replicated in gl.h */

/* src texel formats */
#define SRC_TEXEL_I_8		1
#define SRC_TEXEL_IA_8		2
#define SRC_TEXEL_RGB_8		3
#define SRC_TEXEL_RGBA_8	4
#define SRC_TEXEL_I_16		5
#define SRC_TEXEL_IA_16		6
#define SRC_TEXEL_RGB_16	7
#define SRC_TEXEL_RGBA_16	8

/* dst texel formats */
#define DST_TEXEL_I_16		1
#define DST_TEXEL_I_12A_4	2
#define DST_TEXEL_IA_8		3
#define DST_TEXEL_RGB_5		4
#define DST_TEXEL_RGBA_4	5
#define DST_TEXEL_IA_12		6
#define DST_TEXEL_RGBA_8	7
#define DST_TEXEL_RGB_12	8
#define DST_TEXEL_RGBA_12	9

struct ulamslot { /* User LinearAddressMemory SLOT */

	/* These three fields are Kernel read/write, user read-only */
	long		state;	/* XXX this field and ref. stmts can be removed */
	long		taddr;  /* SKYWR and VGX: address in texture memory; VENICE - bankA offset */
	long		macr;	/*           VGX: imp mac;                   VENICE - bankB offset */
	long		reverse;  /* True if Kernel had to reverse sense of bankA VENICE only */

	/* These fields are User read/write, kernel read-only */
	unsigned long*	vaddr;  /* virtual user address of texture; (SKYWR:NULL for fog)*/
	long		len;	/* length of texture in hardware; given in 32-bit words */
	long		src_len;	/* length of source texture; for video download and
					 * such, the number of bytes going into the GE's
					 * may be different than the number actually put into
					 * texture memory 
					 */

	unsigned long*  vaddr2; /* VENICE has two texture banks; (VENICE only ) */
	long		src_len2;	/* VENICE has two texture banks; (VENICE only ) */
	long		len2;	/* VENICE has two texture banks; (VENICE only ) */

	long		bankA;  /* top level is in bank A */

	/* for on-the-fly texture downloads */
	long		download_style; 
	long		width;
	long		height;
	long		texel_size;
	long		src_texel_format;
	long		dst_texel_format;
	long		src_width;
	long		src_height;
	long		src_eol_offset;
	long 		xsrc;	/* for loads from the framebuffer */
	long		ysrc;

	/*  This field is for GL use only */
	void*		texdef;
};

