#ifndef __UNISTD_H__
#define __UNISTD_H__

#ifdef __cplusplus
extern "C" {
#endif

/* Copyright (C) 1989 Silicon Graphics, Inc. All rights reserved.  */
/* ------------------------------------------------------------------ */
/* | Copyright Unpublished, MIPS Computer Systems, Inc.  All Rights | */
/* | Reserved.  This software contains proprietary and confidential | */
/* | information of MIPS and its suppliers.  Use, disclosure or     | */
/* | reproduction is prohibited without the prior express written   | */
/* | consent of MIPS.                                               | */
/* ------------------------------------------------------------------ */
/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident "$Revision: 1.34 $"

#include <sys/types.h>
#include <time.h>
#include <utime.h>	/* for utime(2) routine. */

/* Symbolic constants for lseek(2) routine: */
#ifndef SEEK_SET		/* also defined in stdio.h */
#define	SEEK_SET	0	/* Set file pointer to "offset" */
#define	SEEK_CUR	1	/* Set file pointer to current plus "offset" */
#define	SEEK_END	2	/* Set file pointer to EOF plus "offset" */
#endif /* !SEEK_SET */

/* Symbolic constants for "access" routine XXX also defined in sys/file.h */
#ifndef R_OK
#define	R_OK	4	/* Test for Read permission */
#define	W_OK	2	/* Test for Write permission */
#define	X_OK	1	/* Test for eXecute permission */
#define	F_OK	0	/* Test for existence of File */
#endif

#define F_ULOCK	0	/* Unlock a previously locked region */
#define F_LOCK	1	/* Lock a region for exclusive use */
#define F_TLOCK	2	/* Test and lock a region for exclusive use */
#define F_TEST	3	/* Test a region for other processes locks */

/* Path names: */
#define	GF_PATH	"/etc/group"	/* Path name of the "group" file */
#define	PF_PATH	"/etc/passwd"	/* Path name of the "passwd" file */


#ifndef _POSIX_VERSION
#define _POSIX_VERSION		198808L	/* 4-digit year, 2-digit month */
#endif
#define _POSIX_JOB_CONTROL	1	/* nonzero (TRUE) */
#define _POSIX_SAVED_IDS	1	/* nonzero (TRUE) */

/* The following allows individual disabling of special control chars. */
#define _POSIX_VDISABLE	0xff

/*
 * Symbolic values associated with stdin, stdout, stderr:
 */
#define STDIN_FILENO	0
#define STDOUT_FILENO	1
#define STDERR_FILENO	2

/*
 * Arguments for the POSIX sysconf() and pathconf() system calls:
 */
#include <sys/syssgi.h>

#define	_SC_ARG_MAX		_INT_ARG_MAX
#define	_SC_CHILD_MAX		_INT_CHILD_MAX
#define	_SC_CLK_TCK		_INT_CLK_TCK
#define	_SC_NGROUPS_MAX		_INT_NGROUPS_MAX
#define	_SC_OPEN_MAX		_INT_OPEN_MAX
#define	_SC_JOB_CONTROL		_INT_JOB_CONTROL
#define	_SC_SAVED_IDS		_INT_SAVED_IDS
#define	_SC_VERSION		_INT_VERSION

#define _PC_LINK_MAX		_INT_LINK_MAX
#define _PC_MAX_CANON		_INT_MAX_CANON
#define _PC_MAX_INPUT		_INT_MAX_INPUT
#define _PC_NAME_MAX		_INT_NAME_MAX
#define _PC_PATH_MAX		_INT_PATH_MAX
#define _PC_PIPE_BUF		_INT_PIPE_BUF
#define _PC_CHOWN_RESTRICTED	_INT_CHOWN_RESTRICTED
#define _PC_NO_TRUNC		_INT_NO_TRUNC
#define _PC_VDISABLE		_INT_VDISABLE


extern pid_t	fork(void);
extern int	execl(const char *, const char *, ...);
extern int	execv(const char *, const char **);
extern int	execle(const char *, const char *, ...);
extern int	execve(const char *, const char **, const char **);
extern int	execlp(const char *, const char *, ...);
extern int	execvp(const char *, const char **);
extern void	_exit(int);
extern uint	alarm(uint);
extern int	pause(void);
extern uint	sleep(uint);

extern pid_t	getpid(void);
extern pid_t	getppid(void);
extern uid_t	getuid(void);
extern uid_t	geteuid(void);
extern gid_t	getgid(void);
extern gid_t	getegid(void);
extern int	setuid(uid_t);
extern int	setgid(gid_t);
extern int	setreuid(int, int);
extern int	setregid(int, int);
extern int	getgroups(int, gid_t *);
extern char *	getlogin(void);
extern char *	cuserid(char *);
extern pid_t	setsid(void);
extern int	setpgid(pid_t, pid_t);
#ifdef _BSD_COMPAT 
extern int	getpgrp(int);
extern int      setpgrp(int, int);
extern int      setgroups(int, int *);
extern int      initgroups(char *, int);
#else
extern pid_t	getpgrp(void);
extern int      setpgrp(void);
extern int      setgroups(int, gid_t *);
extern int      initgroups(char *, gid_t);
#endif /* _BSD_COMPAT */
extern int      BSDsetpgrp(int, int);
extern int      BSDsetgroups(int, int *);
extern int      BSDinitgroups(char *, int);
extern char *	getenv(const char *);
extern char *	ctermid(char *);
extern char *	ttyname(int);
extern int	isatty(int);
extern long	sysconf(int);

extern int	chdir(const char *);
extern char *	getcwd(char *, int);
extern char *	getwd(char *);
extern int	link(const char *, const char *);
extern int	unlink(const char *);
extern int	symlink(const char *, const char *);
extern int	rmdir(const char *);
extern int	rename(const char *, const char *);
extern int	access(const char *, int);
extern int	chown(const char *, uid_t, gid_t);
extern int	fchown(int, uid_t, gid_t);
extern int	utime(const char *, struct utimbuf *);
extern long	pathconf(const char *, int);
extern long	fpathconf(int, int);
extern int	pipe(int *);
extern int	dup(int);
extern int	dup2(int, int);
extern int	close(int);
extern int	read(int, void *, unsigned int);
extern int	write(int, const void *, unsigned int);
extern off_t	lseek(int, off_t, int);
extern int	lockf(int, int, long);

extern pid_t	tcgetpgrp(int);
extern int	tcsetpgrp(int, pid_t);

extern int	truncate(char *, long);
extern int	ftruncate(int, long);

#ifdef __EXTENSIONS__

/* _daemonize(3C) flags */
#define	_DF_NOFORK	0x1	/* don't fork */
#define	_DF_NOCHDIR	0x2	/* don't chdir to / */
#define	_DF_NOCLOSE	0x4	/* close no files */

extern int	_daemonize(int, int, int, int);
extern char *	_getpty(int *, int, mode_t, int);

#endif

#ifdef __cplusplus
}
#endif

#endif /* !__UNISTD_H__ */
